@@ -3,10 +3,10 @@
 #' @description
 #'
 #' These functions are equivalent to base functions [base::stop()],
-#' [base::warning()] and [base::message()], but make it easy to supply
+#' [base::warning()], and [base::message()], but make it easy to supply
 #' condition metadata:
 #'
-#' * Supply `.subclass` to create a classed condition. Typed
+#' * Supply `class` to create a classed condition. Typed
 #'   conditions can be captured or handled selectively, allowing for
 #'   finer-grained error handling.
 #'
@@ -49,29 +49,16 @@
 #' signalled with a `"resume"` restart. This is however not
 #' guaranteed.
 #'
-#'
-#' @section Lifecycle:
-#'
-#' These functions were changed in rlang 0.3.0 to take condition
-#' metadata with `...`. Consequently:
-#'
-#' * All arguments were renamed to be prefixed with a dot, except for
-#'   `type` which was renamed to `.subclass`.
-#' * `.call` (previously `call`) can no longer be passed positionally.
-#'
 #' @inheritParams cnd
 #' @param message The message to display.
 #'
-#'   Experimental: Can also be a named character vector, in which case
-#'   the message is assembled as a list of bullets. See
-#'   [cnd_message()] to learn how names control the bulleted output.
+#'   If not supplied, it is expected that the message is generated
+#'   lazily through [conditionMessage()][cnd_message]. In that case,
+#'   `class` must be supplied. Only `inform()` allows empty messages
+#'   as it is occasionally useful to build user output incrementally.
 #' @param class Subclass of the condition. This allows your users
 #'   to selectively handle the conditions signalled by your functions.
 #' @param ... Additional data to be stored in the condition object.
-#' @param call Defunct as of rlang 0.4.0. Storing the full
-#'   backtrace is now preferred to storing a simple call.
-#' @param msg,type These arguments were renamed to `message` and
-#'   `.subclass` and are defunct as of rlang 0.4.0.
 #' @param .subclass This argument was renamed to `class` in rlang
 #'   0.4.2.  It will be deprecated in the next major version. This is
 #'   for consistency with our conventions for class constructors
@@ -109,17 +96,20 @@
 #'   }
 #' )
 #'
-#' # If you call low-level APIs it is good practice to catch technical
-#' # errors and rethrow them with a more meaningful message. Pass on
-#' # the caught error as `parent` to get a nice decomposition of
-#' # errors and backtraces:
+#' # If you call low-level APIs it is good practice to handle
+#' # technical errors and rethrow them with a more meaningful
+#' # message. Always prefer doing this from `withCallinghandlers()`
+#' # rather than `tryCatch()` because the former preserves the stack
+#' # on error and makes it possible for users to use `recover()`.
 #' file <- "http://foo.bar/baz"
-#' tryCatch(
+#' try(withCallinghandlers(
 #'   download(file),
 #'   error = function(err) {
 #'     msg <- sprintf("Can't download `%s`", file)
 #'     abort(msg, parent = err)
-#' })
+#' }))
+#' # Note how we supplied the parent error to `abort()` as `parent` to
+#' # get a decomposition of error messages across error contexts.
 #'
 #' # Unhandled errors are saved automatically by `abort()` and can be
 #' # retrieved with `last_error()`. The error prints with a simplified
@@ -132,29 +122,35 @@
 #'
 #' }
 #' @export
-abort <- function(message = "",
+abort <- function(message = NULL,
                   class = NULL,
                   ...,
                   trace = NULL,
-                  call,
                   parent = NULL,
-                  msg, type, .subclass) {
-  validate_signal_args(msg, type, call, .subclass)
+                  .subclass) {
+  validate_signal_args(.subclass)
 
-  if (is_null(trace) && is_null(peek_option("rlang__disable_trace_capture"))) {
+  if (is_null(trace) && is_null(peek_option("rlang:::disable_trace_capture"))) {
     # Prevents infloops when rlang throws during trace capture
-    local_options("rlang__disable_trace_capture" = TRUE)
+    with_options("rlang:::disable_trace_capture" = TRUE, {
+      trace <- trace_back()
 
-    trace <- trace_back()
+      # Remove throwing context. Especially important when rethrowing
+      # from a condition handler because in that case there are a
+      # bunch of irrelevant frames in the trailing branch of the call
+      # stack. We want to display the call tree branch that is
+      # relevant to users in simplified backtraces.
+      if (is_null(parent)) {
+        context <- trace_length(trace)
+      } else {
+        context <- trace_capture_depth(trace)
+      }
 
-    if (is_null(parent)) {
-      context <- trace_length(trace)
-    } else {
-      context <- find_capture_context(3L)
-    }
-    trace <- trace_trim_context(trace, context)
+      trace <- trace_trim_context(trace, context)
+    })
   }
 
+  message <- validate_signal_message(message, class)
   message <- collapse_cnd_message(message)
 
   cnd <- error_cnd(class,
@@ -167,7 +163,7 @@ abort <- function(message = "",
 }
 
 signal_abort <- function(cnd) {
-  if (is_true(peek_option("rlang_force_unhandled_error"))) {
+  if (is_true(peek_option("rlang:::force_unhandled_error"))) {
     # Fall back with the full rlang error
     fallback <- cnd
   } else {
@@ -185,21 +181,34 @@ signal_abort <- function(cnd) {
   # Save the unhandled error for `rlang::last_error()`.
   last_error_env$cnd <- cnd
 
-  # Generate the error message, possibly with a backtrace or reminder
-  fallback$message <- paste_line(
-    conditionMessage(cnd),
-    format_onerror_backtrace(cnd)
-  )
+  if (is_interactive()) {
+    # Generate the error message, possibly with a backtrace or reminder
+    fallback$message <- paste_line(
+      conditionMessage(cnd),
+      format_onerror_backtrace(cnd)
+    )
+    fallback$rlang_entraced <- TRUE
+  } else {
+    file <- peek_option("rlang:::error_pipe") %||% stderr()
+    msg <- conditionMessage(cnd)
+    fallback$message <- msg
+
+    cat("Error: ", msg, "\n", sep = "", file = file)
+
+    # Print the backtrace eagerly in non-interactive sessions because
+    # the length of error messages is limited (#856)
+    cat(format_onerror_backtrace(cnd), "\n", sep = "", file = file)
+
+    # Turn off the regular error printing to avoid printing the error
+    # twice
+    local_options(show.error.messages = FALSE)
+  }
 
   stop(fallback)
 }
 
-trace_trim_context <- function(trace, frame = caller_env()) {
-  if (is_environment(frame)) {
-    idx <- detect_index(trace$ids, identical, env_label(frame))
-  } else if (is_scalar_integerish(frame)) {
-    idx <- frame
-  } else {
+trace_trim_context <- function(trace, idx) {
+  if (!is_scalar_integerish(idx)) {
     abort("`frame` must be a frame environment or index")
   }
 
@@ -211,17 +220,38 @@ trace_trim_context <- function(trace, frame = caller_env()) {
   trace
 }
 
-# Assumes we're called from an exiting handler. Need to
-find_capture_context <- function(n = 3L) {
-  parent <- orig <- sys.parent(n)
-  call <- sys.call(parent)
+# Assumes we're called from a calling or exiting handler
+trace_capture_depth <- function(trace) {
+  calls <- trace$calls
+  default <- length(calls)
 
-  try_catch_n <- detect_index(sys.calls(), is_call, "tryCatch", .right = TRUE)
-  if (try_catch_n == 0L) {
-    sys.frame(sys.parent(n))
-  } else {
-    sys.frame(try_catch_n)
+  if (length(calls) <= 3L) {
+    return(default)
   }
+
+  # withCallingHandlers()
+  wch_calls <- calls[seq2(length(calls) - 3L, length(calls) - 1L)]
+  if (is_call(wch_calls[[1]], "signal_abort") &&
+      is_call(wch_calls[[2]], "signalCondition") &&
+      is_call(wch_calls[[3]]) && is_function(wch_calls[[3]][[1]])) {
+    return(length(calls) - 4L)
+  }
+
+  exiting_call <- quote(value[[3L]](cond))
+  exiting_n <- detect_index(calls, identical, exiting_call, .right = TRUE)
+
+  # tryCatch()
+  if (exiting_n != 0L) {
+    try_catch_calls <- calls[seq_len(exiting_n - 1L)]
+    try_catch_n <- detect_index(try_catch_calls, is_call, "tryCatch", .right = TRUE)
+    if (try_catch_n != 0L) {
+      return(try_catch_n)
+    } else {
+      return(default)
+    }
+  }
+
+  default
 }
 
 #' Display backtrace on error
@@ -266,8 +296,25 @@ find_capture_context <- function(n = 3L) {
 #' # stop("foo")
 NULL
 
+# Whenever the backtrace-on-error format is changed, the version in
+# `inst/backtrace-ver` and in `tests/testthat/helper-rlang.R` must be
+# bumped. This way `devtools::test()` will skip the tests that require
+# the dev version to be installed locally.
 format_onerror_backtrace <- function(cnd) {
   trace <- cnd$trace
+
+  # Show backtrace of oldest parent
+  while (is_condition(cnd$parent)) {
+    cnd <- cnd$parent
+    if (!is_null(cnd$trace)) {
+      trace <- cnd$trace
+    }
+  }
+
+  if (is_null(trace) || !trace_length(trace)) {
+    return(NULL)
+  }
+
   show_trace <- show_trace_p()
 
   opts <- c("none", "reminder", "branch", "collapse", "full")
@@ -280,6 +327,14 @@ format_onerror_backtrace <- function(cnd) {
   if (show_trace == "none") {
     return(NULL)
   }
+  if (show_trace == "reminder") {
+    if (is_interactive()) {
+      reminder <- silver("Run `rlang::last_error()` to see where the error occurred.")
+    } else {
+      reminder <- NULL
+    }
+    return(reminder)
+  }
 
   if (show_trace == "branch") {
     max_frames <- 10L
@@ -293,30 +348,9 @@ format_onerror_backtrace <- function(cnd) {
     show_trace
   )
 
-  if (simplify == "none") {
-    # Show full backtrace including for parent errors
-    return(format(cnd, simplify = "none"))
-  }
-
-  backtrace_lines <- format(trace, simplify = simplify, max_frames = max_frames)
-
-  # Backtraces of size 0 and 1 are uninteresting
-  if (length(backtrace_lines) <= 1L) {
-    return(NULL)
-  }
-
-  if (show_trace == "reminder") {
-    if (is_interactive()) {
-      reminder <- silver("Run `rlang::last_error()` to see where the error occurred.")
-    } else {
-      reminder <- NULL
-    }
-    return(reminder)
-  }
-
   paste_line(
     "Backtrace:",
-    backtrace_lines
+    format(trace, simplify = simplify, max_frames = max_frames)
   )
 }
 
