@@ -90,16 +90,39 @@ validate_cnd_signal_args <- function(cnd, .cnd, .mufflable,
 }
 
 #' @rdname abort
+#' @param .frequency How frequently should the warning or message be
+#'   displayed? By default (`"always"`) it is displayed at each
+#'   time. If `"regularly"`, it is displayed once every 8 hours. If
+#'   `"once"`, it is displayed once per session.
+#' @param .frequency_id A unique identifier for the warning or
+#'   message. This is used when `.frequency` is supplied to recognise
+#'   recurring conditions. This argument must be supplied if
+#'   `.frequency` is not set to `"always"`.
 #' @export
-warn <- function(message, class = NULL, ..., call, msg, type, .subclass) {
-  validate_signal_args(msg, type, call, .subclass)
+warn <- function(message = NULL,
+                 class = NULL,
+                 ...,
+                 .frequency = c("always", "regularly", "once"),
+                 .frequency_id = NULL,
+                 .subclass) {
+  validate_signal_args(.subclass)
 
+  message <- validate_signal_message(message, class)
   message <- collapse_cnd_message(message)
+
+  .frequency <- arg_match(.frequency, c("always", "regularly", "once"))
+
+  if (needs_signal(.frequency, .frequency_id, warning_freq_env)) {
+    message <- add_message_freq(message, .frequency, "warning")
+  } else {
+    return(invisible(NULL))
+  }
+
   cnd <- warning_cnd(class, ..., message = message)
   warning(cnd)
 }
 #' @rdname abort
-#' @param file Where the message is printed. This should be a
+#' @param .file Where the message is printed. This should be a
 #'   connection or character string which will be passed to [cat()].
 #'
 #'   By default, `inform()` prints to standard output in interactive
@@ -107,25 +130,37 @@ warn <- function(message, class = NULL, ..., call, msg, type, .subclass) {
 #'   messages distinctly from warnings and errors, and R scripts can
 #'   still filter out the messages easily by redirecting `stderr`.
 #' @export
-inform <- function(message,
+inform <- function(message = NULL,
                    class = NULL,
                    ...,
-                   file = NULL,
-                   call,
-                   msg,
-                   type,
+                   .file = NULL,
+                   .frequency = c("always", "regularly", "once"),
+                   .frequency_id = NULL,
                    .subclass) {
-  validate_signal_args(msg, type, call, .subclass)
+  validate_signal_args(.subclass)
 
+  message <- message %||% ""
   message <- collapse_cnd_message(message)
+  message <- paste0(message, "\n")
+
+  .frequency <- arg_match(.frequency, c("always", "regularly", "once"))
+
+  if (needs_signal(.frequency, .frequency_id, message_freq_env)) {
+    message <- add_message_freq(message, .frequency, "message")
+  } else {
+    return(invisible(NULL))
+  }
+
   cnd <- message_cnd(class, ..., message = message)
 
   withRestarts(muffleMessage = function() NULL, {
     signalCondition(cnd)
 
-    file <- file %||% if (is_interactive()) stdout() else stderr()
-    cat_line(message, file = file)
+    .file <- .file %||% if (is_interactive()) stdout() else stderr()
+    cat(message, file = .file)
   })
+
+  invisible()
 }
 #' @rdname abort
 #' @export
@@ -138,16 +173,7 @@ signal <- function(message, class, ..., .subclass) {
   cnd_signal(cnd)
 }
 
-validate_signal_args <- function(msg, type, call, subclass, env = caller_env()) {
-  if (!missing(msg)) {
-    stop_defunct("`msg` has been renamed to `message` and is deprecated as of rlang 0.3.0")
-  }
-  if (!missing(type)) {
-    stop_defunct("`type` has been renamed to `.subclass` and is deprecated as of rlang 0.3.0")
-  }
-  if (!missing(call)) {
-    stop_defunct("`call` is deprecated as of rlang 0.3.0")
-  }
+validate_signal_args <- function(subclass, env = caller_env()) {
   if (!missing(subclass)) {
     deprecate_subclass(subclass, env = env)
   }
@@ -162,3 +188,63 @@ deprecate_subclass <- function(subclass, env = caller_env()) {
 interrupt <- function() {
   .Call(rlang_interrupt)
 }
+
+validate_signal_message <- function(msg, class) {
+  if (is_null(msg)) {
+    if (is_null(class)) {
+      abort("Either `message` or `class` must be supplied.")
+    }
+    msg <- ""
+  }
+
+  msg
+}
+
+
+warning_freq_env <- new.env(parent = emptyenv())
+message_freq_env <- new.env(parent = emptyenv())
+
+needs_signal <- function(frequency, id, env) {
+  if (is_string(frequency, "always")) {
+    return(TRUE)
+  }
+  if (is_true(peek_option("rlang:::message_always"))) {
+    return(TRUE)
+  }
+
+  if (is_null(id)) {
+    abort("`.frequency_id` should be supplied with `.frequency`.")
+  }
+
+  sentinel <- env[[id]]
+  if (is_null(sentinel)) {
+    env_poke(env, id, Sys.time())
+    return(TRUE)
+  }
+
+  if (is_string(frequency, "once")) {
+    return(FALSE)
+  }
+
+  if (!inherits(sentinel, "POSIXct")) {
+    stop_internal("needs_signal", "Expected `POSIXct` value.")
+  }
+
+  # Signal every 8 hours
+  (Sys.time() - sentinel) > (8 * 60 * 60)
+}
+
+add_message_freq <- function(message, frequency, type) {
+  if (is_string(frequency, "always")) {
+    return(message)
+  }
+
+  if (is_string(frequency, "regularly")) {
+    info <- silver("This %s is displayed once every 8 hours.")
+  } else {
+    info <- silver("This %s is displayed once per session.")
+  }
+  info <- sprintf(info, type)
+
+  paste_line(message, info)
+}
