@@ -92,6 +92,21 @@
 #' ```
 #'
 #'
+#' @section Stack semantics of `eval_tidy()`:
+#'
+#' `eval_tidy()` has different stack semantics than [base::eval()]:
+#'
+#' - Functions that require the evaluation environment to correspond
+#'   to a frame on the call stack do not work. This is why `return()`
+#'   called from a quosure does not work.
+#'
+#' - The mask environment creates a new branch in the call tree
+#'   defined by `sys.parent()` (you can visualise it in a [browser()]
+#'   session with `lobstr::cst()`).
+#'
+#' See also [eval_bare()] for more information about these differences.
+#'
+#'
 #' @section Life cycle:
 #'
 #' **rlang 0.3.0**
@@ -165,43 +180,47 @@ names.rlang_fake_data_pronoun <- function(...) NULL
 #' @export
 print.rlang_fake_data_pronoun <- function(...) cat_line("<pronoun>")
 
-#' Data pronoun for tidy evaluation
+#' Data pronouns for tidy evaluation
 #'
 #' @description
 #'
-#' This pronoun allows you to be explicit when you refer to an object
-#' inside the data. Referring to the `.data` pronoun rather than to
-#' the original data frame has several advantages:
+#' These pronouns allow you to be explicit about where to find objects
+#' when programming with data masked functions.
 #'
-#' * It makes it easy to refer to column names stored as strings. If
-#'   `var` contains the column `"height"`, the pronoun will subset that
-#'   column:
+#' ```
+#' m <- 10
+#' mtcars %>% mutate(disp = .data$disp * .env$m)
+#' ```
 #'
-#'     ```
-#'     var <- "height"
-#'     dplyr::summarise(df, mean(.data[[var]]))
-#'     ```
+#' * `.data` retrieves data-variables from the data frame.
+#' * `.env` retrieves env-variables from the environment.
 #'
-#'   The index variable `var` is [unquoted][nse-force], which
-#'   ensures a column named `var` in the data frame cannot mask it.
-#'   This makes the pronoun safe to use in functions and packages.
+#' Because the lookup is explicit, there is no ambiguity between both
+#' kinds of variables. Compare:
 #'
-#' * Sometimes a computation is not about the whole data but about a
-#'   subset. For example if you supply a grouped data frame to a dplyr
-#'   verb, the `.data` pronoun contains the group subset.
+#' ```
+#' disp <- 10
+#' mtcars %>% mutate(disp = .data$disp * .env$disp)
+#' mtcars %>% mutate(disp = disp * disp)
+#' ```
 #'
-#' * It lets dplyr know that you're referring to a column from the
-#'   data which is helpful to generate correct queries when the source
-#'   is a database.
+#' The `.data` object exported from rlang is also useful to import in
+#' your package namespace to avoid a `R CMD check` note when referring
+#' to objects from the data mask.
 #'
-#' The `.data` object exported here is useful to import in your
-#' package namespace to avoid a `R CMD check` note when referring to
-#' objects from the data mask.
+#' Note that `.data` is only a pronoun, it is not a real data
+#' frame. This means that you can't take its names or map a function
+#' over the contents of `.data`. Similarly, `.env` is not an actual R
+#' environment. For instance, it doesn't have a parent and the
+#' subsetting operators behave differently.
 #'
 #' @name tidyeval-data
 #' @format NULL
 #' @export
 .data <- structure(list(), class = "rlang_fake_data_pronoun")
+#' @rdname tidyeval-data
+#' @export
+.env <- .data
 
 
 #' Create a data mask
@@ -299,35 +318,7 @@ print.rlang_fake_data_pronoun <- function(...) cat_line("<pronoun>")
 #' Consequently, all masking data should be contained between the
 #' bottom and top environment of the data mask.
 #'
-#' @section Life cycle:
-#'
-#' The `parent` argument no longer has any effect and is defunct as of
-#' rlang 0.4.0. The parent of the data mask is determined from either:
-#'
-#'   * The `env` argument of `eval_tidy()`
-#'   * Quosure environments when applicable
-#'
-#' Passing environments to `as_data_mask()` is deprecated as of rlang
-#' 0.3.0. Please use `new_data_mask()` instead.
-#'
-#' **rlang 0.2.0**
-#'
-#' In early versions of rlang data masks were called overscopes. We
-#' think data mask is a more natural name in R. It makes reference to
-#' masking in the search path which occurs through the same mechanism
-#' (in technical terms, lexical scoping with hierarchically nested
-#' environments). We say that objects from user data mask objects
-#' in the current environment.
-#'
-#' Following this change in terminology, `as_overscope()` and
-#' `new_overscope()` were deprecated in rlang 0.2.0 in favour of
-#' `as_data_mask()` and `new_data_mask()`.
-#'
 #' @param data A data frame or named vector of masking data.
-#' @param parent Deprecated. This argument no longer has any effect.
-#'   The parent of the data mask is determined from either:
-#'   * The `env` argument of `eval_tidy()`
-#'   * Quosure environments when applicable
 #' @return A data mask that you can supply to [eval_tidy()].
 #'
 #' @export
@@ -395,16 +386,7 @@ print.rlang_fake_data_pronoun <- function(...) cat_line("<pronoun>")
 #'
 #' # Now we can reference the values with the pronouns:
 #' eval_tidy(quote(c(.data$a, .data$b, .data$c)), data = mask)
-as_data_mask <- function(data, parent = NULL) {
-  if (!is_null(parent)) {
-    stop_defunct(paste_line(
-      "The `parent` argument of `as_data_mask()` is deprecated.",
-      "The parent of the data mask is determined from either:",
-      "",
-      "  * The `env` argument of `eval_tidy()`",
-      "  * Quosure environments when applicable"
-    ))
-  }
+as_data_mask <- function(data) {
   .Call(rlang_as_data_mask, data)
 }
 #' @rdname as_data_mask
@@ -429,16 +411,7 @@ as_data_pronoun <- function(data) {
 #'   eval engine and should be considered undetermined. Never make
 #'   assumption about the parent of `top`.
 #' @export
-new_data_mask <- function(bottom, top = bottom, parent = NULL) {
-  if (!is_null(parent)) {
-    stop_defunct(paste_line(
-      "The `parent` argument of `new_data_mask()` is deprecated.",
-      "The parent of the data mask is determined from either:",
-      "",
-      "  * The `env` argument of `eval_tidy()`",
-      "  * Quosure environments when applicable"
-    ))
-  }
+new_data_mask <- function(bottom, top = bottom) {
   .Call(rlang_new_data_mask, bottom, top)
 }
 
@@ -452,7 +425,7 @@ new_data_mask <- function(bottom, top = bottom, parent = NULL) {
 }
 data_pronoun_get <- function(x, nm) {
   if (!is_string(nm)) {
-    abort("Must subset the data pronoun with a string")
+    abort("Must subset the data pronoun with a string.")
   }
   mask <- .subset2(x, 1)
   .Call(rlang_data_pronoun_get, mask, sym(nm))
@@ -472,7 +445,7 @@ abort_data_pronoun <- function(nm) {
 }
 ctxt_pronoun_get <- function(x, nm) {
   if (!is_string(nm)) {
-    abort("Must subset the context pronoun with a string")
+    abort("Must subset the context pronoun with a string.")
   }
   eval_bare(sym(nm), x)
 }
