@@ -88,10 +88,10 @@ trace_back <- function(top = NULL, bottom = NULL) {
   calls <- map2(calls, seq_along(calls), maybe_add_namespace)
 
   parents <- normalise_parents(parents)
-  ids <- map_chr(frames, env_label)
 
-  trace <- new_trace(calls, parents, ids)
-  trace <- trace_trim_env(trace, top)
+  trace <- new_trace(calls, parents)
+  trace <- add_winch_trace(trace)
+  trace <- trace_trim_env(trace, frames, top)
 
   trace
 }
@@ -237,7 +237,7 @@ normalise_parents <- function(parents) {
   parents
 }
 
-new_trace <- function(calls, parents, ids, indices = NULL) {
+new_trace <- function(calls, parents, indices = NULL) {
   indices <- indices %||% seq_along(calls)
 
   n <- length(calls)
@@ -251,10 +251,11 @@ new_trace <- function(calls, parents, ids, indices = NULL) {
     list(
       calls = calls,
       parents = parents,
-      ids = ids,
       indices = indices
     ),
-    class = "rlang_trace"
+    class = "rlang_trace",
+    # Increment this number when the internal format for the class changes
+    version = 1L
   )
 }
 
@@ -263,6 +264,28 @@ trace_reset_indices <- function(trace) {
   trace
 }
 
+# Can't use new_environment() here
+winch_available_env <- new.env(parent = emptyenv())
+
+add_winch_trace <- function(trace) {
+  avail <- winch_available_env$installed
+  if (is_null(avail)) {
+    avail <- rlang::is_installed("winch")
+    winch_available_env$installed <- avail
+  }
+
+  if (!avail) {
+    return(trace)
+  }
+
+  use_winch <- peek_option("rlang_trace_use_winch") %||% FALSE
+  if (!is_true(as.logical(use_winch))) {
+    return(trace)
+  }
+
+  winch::winch_add_trace_back(trace)
+}
+
 # Methods -----------------------------------------------------------------
 
 # For internal use only
@@ -271,10 +294,9 @@ c.rlang_trace <- function(...) {
 
   calls <- flatten(map(traces, `[[`, "calls"))
   parents <- flatten_int(map(traces, `[[`, "parents"))
-  ids <- flatten_chr(map(traces, `[[`, "ids"))
   indices <- flatten_int(map(traces, `[[`, "indices"))
 
-  new_trace(calls, parents, ids, indices)
+  new_trace(calls, parents, indices)
 }
 
 #' @export
@@ -434,7 +456,7 @@ trace_length <- function(trace) {
 
 trace_subset <- function(x, i) {
   if (!length(i)) {
-    return(new_trace(list(), int(), list()))
+    return(new_trace(list(), int()))
   }
   stopifnot(is_integerish(i))
 
@@ -449,7 +471,6 @@ trace_subset <- function(x, i) {
   new_trace(
     calls = x$calls[i],
     parents = parents,
-    ids = x$ids[i],
     indices = x$indices[i]
   )
 }
@@ -508,20 +529,20 @@ parents_indices <- function(i, parents) {
 
 # Trimming ----------------------------------------------------------------
 
-trace_trim_env <- function(x, to = NULL) {
+trace_trim_env <- function(x, frames, to) {
   to <- to %||% peek_option("rlang_trace_top_env")
 
   if (is.null(to)) {
     return(x)
   }
 
-  is_top <- x$ids == env_label(to)
+  is_top <- map_lgl(frames, is_reference, to)
   if (!any(is_top)) {
     return(x)
   }
 
-  start <- last(which(is_top)) + 1
-  end <- length(x$ids)
+  start <- last(which(is_top)) + 1L
+  end <- trace_length(x)
 
   trace_subset(x, seq2(start, end))
 }
@@ -613,6 +634,8 @@ has_pipe_pointer <- function(x) {
 
 # Assumes a backtrace branch with collapsed pipe
 branch_uncollapse_pipe <- function(trace) {
+  orig <- trace
+
   while (idx <- detect_index(trace$calls, has_pipe_pointer)) {
     trace_before <- trace_subset(trace, seq2(1L, idx - 1L))
     trace_after <- trace_subset(trace, seq2(idx + 2L, trace_length(trace)))
@@ -639,12 +662,9 @@ branch_uncollapse_pipe <- function(trace) {
     parent <- trace$parents[[idx]]
     pipe_parents <- seq(parent, parent + pointer - 1L)
 
-    # Assign the pipe frame as dummy ids for uncollapsed frames
-    pipe_ids <- rep(trace$ids[idx], pointer)
-
     # This assigns the same frame number to all pipe calls that have
     # already returned
-    n_collapsed <- attr(pipe, "collapsed")
+    n_collapsed <- attr(pipe, "collapsed") %||% 0L
     pipe_indices <- rep(idx, length(pipe_calls) - 1L)
     pipe_indices <- c(pipe_indices, idx + n_collapsed + 1L)
 
@@ -654,12 +674,12 @@ branch_uncollapse_pipe <- function(trace) {
 
     trace$calls <- c(trace_before$calls, pipe_calls, trace_after$calls)
     trace$parents <- c(trace_before$parents, pipe_parents, trace_after$parents)
-    trace$ids <- c(trace_before$ids, pipe_ids, trace_after$ids)
     trace$indices <- c(trace_before$indices, pipe_indices, trace_after$indices)
   }
 
   if (length(unique(lengths(trace))) != 1L) {
-    abort("Internal error: Trace data is not square.")
+    warn("Internal error: Trace data is not square.")
+    return(orig)
   }
 
   trace
@@ -708,6 +728,10 @@ is_uninformative_call <- function(call) {
 
   fn <- call[[1]]
 
+  if (is_winch_frame(fn)) {
+    return(TRUE)
+  }
+
   # Inlined functions occur with active bindings
   if (is_function(fn)) {
     return(TRUE)
@@ -724,6 +748,22 @@ is_uninformative_call <- function(call) {
   FALSE
 }
 
+# To be replaced with a more structured way of disabling frames in
+# various displays
+is_winch_frame <- function(call) {
+  if (!is_call(call, "::")) {
+    return(FALSE)
+  }
+
+  lhs <- call[[2]]
+  if (!is_symbol(lhs)) {
+    return(FALSE)
+  }
+
+  name <- as_string(lhs)
+  grepl("^[/\\\\].+[.]", name)
+}
+
 trace_simplify_collapse <- function(trace) {
   parents <- trace$parents
   path <- int()
@@ -767,21 +807,6 @@ trace_simplify_collapse <- function(trace) {
   trace_subset(trace, rev(path))
 }
 
-# Typically the full trace is held by the child error and the partial
-# trace by the parent
-trace_trim_common <- function(full, partial) {
-  common <- map_lgl(full$ids, `%in%`, partial$ids)
-  out <- trace_subset(full, which(!common))
-
-  # Trim catching context if any
-  calls <- out$calls
-  if (length(calls) && is_call(calls[[1]], c("tryCatch", "with_handlers", "catch_cnd"))) {
-    out <- trace_subset_across(out, -1, 1)
-  }
-
-  out
-}
-
 
 # Printing ----------------------------------------------------------------
 
@@ -871,3 +896,115 @@ trace_root <- function() {
     "x"
   }
 }
+
+#' Trace functions from a set of packages
+#'
+#' @param pkgs One or more package names whose functions should be traced.
+#' @param max_level Maximum nesting level of call stack.
+#' @param regexp Regular expression passed to [base::grepl()] to
+#'   select which functions should be traced. Can be a single string
+#'   or a vector as long as `pkgs`.
+#' @param ... These dots are for future extensions and should be empty.
+#'
+#' @author Iñaki Ucar (ORCID: 0000-0001-6403-5550)
+#' @noRd
+trace_pkgs <- function(pkgs, max_level = Inf, ..., regexp = NULL) {
+  check_dots_empty(...)
+
+  # Avoids namespace loading issues
+  lapply(pkgs, requireNamespace, quietly = TRUE)
+
+  trace_level <- 0
+
+  # Create a thunk because `trace()` sloppily transforms functions into calls
+  tracer <- call2(function() {
+    trace_level <<- trace_level + 1
+
+    if (trace_level > max_level) {
+      return()
+    }
+
+    # Work around sys.foo() weirdness
+    get_fn <- call2(function(fn = sys.function(sys.parent())) fn)
+    fn <- eval(get_fn, envir = parent.frame())
+
+    try(silent = TRUE, {
+      call <- evalq(base::match.call(), envir = parent.frame())
+      call <- call_add_namespace(call, fn)
+
+      indent <- paste0(rep(" ", (trace_level - 1) * 2), collapse = "")
+      line <- paste0(indent, as_label(call))
+      cat(line, "\n")
+    })
+  })
+
+  exit <- call2(function() {
+    trace_level <<- trace_level - 1
+  })
+
+  if (length(regexp) == 1) {
+    regexp <- rep_along(pkgs, regexp)
+  }
+
+  for (i in seq_along(pkgs)) {
+    pkg <- pkgs[[i]]
+    ns <- ns_env(pkg)
+    ns_fns <- names(keep(as.list(ns), is.function))
+
+    if (!is_null(regexp)) {
+      ns_fns <- ns_fns[grepl(regexp[[i]], ns_fns)]
+    }
+
+    suppressMessages(trace(
+      ns_fns,
+      tracer = tracer,
+      exit = exit,
+      print = FALSE,
+      where = ns
+    ))
+
+    message(sprintf(
+      "Tracing %d functions in %s.",
+      length(ns_fns),
+      pkg
+    ))
+  }
+}
+
+call_add_namespace <- function(call, fn) {
+  if (!is.call(call) || !is.symbol(call[[1]])) {
+    return(call)
+  }
+
+  sym <- call[[1]]
+  nm <- as_string(sym)
+
+  if (nm %in% c("::", ":::")) {
+    return(call)
+  }
+
+  env <- environment(fn)
+  top <- topenv(env)
+
+  if (is_reference(env, globalenv())) {
+    prefix <- "global"
+    op <- "::"
+  } else if (is_namespace(top)) {
+    prefix <- ns_env_name(top)
+    if (ns_exports_has(top, nm)) {
+      op <- "::"
+    } else {
+      op <- ":::"
+    }
+  } else {
+    return(call)
+  }
+
+  namespaced_sym <- call(op, as.symbol(prefix), sym)
+  call[[1]] <- namespaced_sym
+  call
+}
+
+is_trace <- function(x) {
+  inherits_any(x, c("rlang_trace", "rlib_trace"))
+}
