@@ -24,12 +24,12 @@ static struct rlang_mask_info mask_info(sexp* mask) {
   sexp* flag;
 
   flag = r_env_find_anywhere(mask, data_mask_flag_sym);
-  if (flag != r_unbound_sym) {
+  if (flag != r_syms_unbound) {
     return (struct rlang_mask_info) { flag, RLANG_MASK_DATA };
   }
 
   flag = r_env_find_anywhere(mask, quo_mask_flag_sym);
-  if (flag != r_unbound_sym) {
+  if (flag != r_syms_unbound) {
     return (struct rlang_mask_info) { flag, RLANG_MASK_QUOSURE };
   }
 
@@ -45,7 +45,7 @@ static sexp* rlang_new_data_pronoun(sexp* mask) {
   sexp* pronoun = KEEP(r_new_vector(r_type_list, 1));
 
   r_list_poke(pronoun, 0, mask);
-  r_poke_attribute(pronoun, r_class_sym, data_pronoun_class);
+  r_poke_attribute(pronoun, r_syms_class, data_pronoun_class);
 
   FREE(1);
   return pronoun;
@@ -53,7 +53,7 @@ static sexp* rlang_new_data_pronoun(sexp* mask) {
 static sexp* rlang_new_ctxt_pronoun(sexp* top) {
   sexp* pronoun = KEEP(r_new_environment(r_env_parent(top), 0));
 
-  r_poke_attribute(pronoun, r_class_sym, ctxt_pronoun_class);
+  r_poke_attribute(pronoun, r_syms_class, ctxt_pronoun_class);
 
   FREE(1);
   return pronoun;
@@ -62,7 +62,7 @@ static sexp* rlang_new_ctxt_pronoun(sexp* top) {
 void poke_ctxt_env(sexp* mask, sexp* env) {
   sexp* ctxt_pronoun = r_env_find(mask, data_mask_env_sym);
 
-  if (ctxt_pronoun == r_unbound_sym) {
+  if (ctxt_pronoun == r_syms_unbound) {
     r_abort("Internal error: Can't find context pronoun in data mask");
   }
 
@@ -143,7 +143,7 @@ static sexp* tilde_fn = NULL;
 static sexp* restore_mask_fn = NULL;
 
 static void on_exit_restore_lexical_env(sexp* mask, sexp* old, sexp* frame) {
-  sexp* fn = KEEP(r_duplicate(restore_mask_fn, true));
+  sexp* fn = KEEP(r_clone(restore_mask_fn));
 
   sexp* env = KEEP(r_new_environment(r_base_env, 2));
   r_env_poke(env, mask_sym, mask);
@@ -181,7 +181,7 @@ sexp* rlang_new_data_mask(sexp* bottom, sexp* top) {
 
   sexp* ctxt_pronoun = KEEP(rlang_new_ctxt_pronoun(top));
 
-  r_env_poke(data_mask, r_tilde_sym, tilde_fn);
+  r_env_poke(data_mask, r_syms_tilde, tilde_fn);
   r_env_poke(data_mask, data_mask_flag_sym, data_mask);
   r_env_poke(data_mask, data_mask_env_sym, ctxt_pronoun);
   r_env_poke(data_mask, data_mask_top_env_sym, top);
@@ -210,15 +210,18 @@ static sexp* mask_find(sexp* env, sexp* sym) {
     top_env = env;
   }
   int n_kept = 0;
-  KEEP_N(top_env, n_kept); // Help rchk
+  KEEP_N(top_env, n_kept);
 
   sexp* cur = env;
   do {
     sexp* obj = r_env_find(cur, sym);
     if (TYPEOF(obj) == PROMSXP) {
-      obj = KEEP_N(r_eval(obj, r_empty_env), n_kept);
+      KEEP(obj);
+      obj = r_eval(obj, r_empty_env);
+      FREE(1);
     }
-    if (obj != r_unbound_sym && !r_is_function(obj)) {
+
+    if (obj != r_syms_unbound && !r_is_function(obj)) {
       FREE(n_kept);
       return obj;
     }
@@ -231,7 +234,7 @@ static sexp* mask_find(sexp* env, sexp* sym) {
   } while (cur != r_empty_env);
 
   FREE(n_kept);
-  return r_unbound_sym;
+  return r_syms_unbound;
 }
 sexp* rlang_data_pronoun_get(sexp* pronoun, sexp* sym) {
   if (r_typeof(pronoun) != r_type_environment) {
@@ -240,7 +243,7 @@ sexp* rlang_data_pronoun_get(sexp* pronoun, sexp* sym) {
 
   sexp* obj = mask_find(pronoun, sym);
 
-  if (obj == r_unbound_sym) {
+  if (obj == r_syms_unbound) {
     sexp* call = KEEP(r_parse("rlang:::abort_data_pronoun(x)"));
     r_eval_with_x(call, r_base_env, sym);
     r_abort("Internal error: .data subsetting should have failed earlier");
@@ -355,7 +358,7 @@ static sexp* base_tilde_eval(sexp* tilde, sexp* quo_env) {
   tilde = KEEP(r_eval(tilde, quo_env));
 
   // Change it back because the result still has the primitive inlined
-  r_node_poke_car(tilde, r_tilde_sym);
+  r_node_poke_car(tilde, r_syms_tilde);
 
   FREE(2);
   return tilde;
@@ -364,7 +367,7 @@ static sexp* base_tilde_eval(sexp* tilde, sexp* quo_env) {
 sexp* env_get_top_binding(sexp* mask) {
   sexp* top = r_env_find(mask, data_mask_top_env_sym);
 
-  if (top == r_unbound_sym) {
+  if (top == r_syms_unbound) {
     r_abort("Internal error: Can't find .top pronoun in data mask");
   }
   if (r_typeof(top) != r_type_environment) {
@@ -380,7 +383,7 @@ static sexp* env_poke_fn = NULL;
 
 sexp* rlang_tilde_eval(sexp* tilde, sexp* current_frame, sexp* caller_frame) {
   // Remove srcrefs from system call
-  r_poke_attribute(tilde, r_srcref_sym, r_null);
+  r_poke_attribute(tilde, r_syms_srcref, r_null);
 
   if (!rlang_is_quosure(tilde)) {
     return base_tilde_eval(tilde, caller_frame);
@@ -445,14 +448,14 @@ sexp* rlang_data_mask_clean(sexp* mask) {
   }
 
   // At this level we only want to remove our own stuff
-  r_env_unbind_all(mask, data_mask_objects_names, false);
+  r_env_unbind_strings(mask, data_mask_objects_names);
 
   // Remove everything in the other levels
   sexp* env = bottom;
   sexp* parent = r_env_parent(top);
   while (env != parent) {
     sexp* nms = KEEP(r_env_names(env));
-    r_env_unbind_names(env, nms, false);
+    r_env_unbind_names(env, nms);
     FREE(1);
     env = r_env_parent(env);
   }
@@ -464,7 +467,7 @@ sexp* rlang_data_mask_clean(sexp* mask) {
 
 static sexp* new_quosure_mask(sexp* env) {
   sexp* mask = KEEP(r_new_environment(env, 3));
-  r_env_poke(mask, r_tilde_sym, tilde_fn);
+  r_env_poke(mask, r_syms_tilde, tilde_fn);
   r_env_poke(mask, quo_mask_flag_sym, mask);
   FREE(1);
   return mask;
