@@ -9,5 +9,10 @@ sexp* rlang_enquo(sexp* sym, sexp* frame);
 
 extern sexp* rlang_ns_env;
 
+ __attribute__((noreturn))
+void never_reached(const char* fn) {
+  r_abort("Internal error in `%s()`: Reached the unreachable.", fn);
+}
+
 
 #endif
