@@ -30,12 +30,14 @@ r_ssize r_as_ssize(sexp* n) {
 }
 
 void r_init_rlang_ns_env();
+void r_init_library_arg();
 void r_init_library_cnd();
 void r_init_library_env();
 void r_init_library_session();
 void r_init_library_squash();
 void r_init_library_stack();
 void r_init_library_sym();
+void r_init_library_vec();
 void r_init_library_vec_chr();
 
 sexp* r_shared_true;
@@ -52,11 +54,13 @@ SEXP r_init_library() {
   r_init_library_sym();  // Needs to be first
 
   r_init_rlang_ns_env();
+  r_init_library_arg();
   r_init_library_cnd();
   r_init_library_env();
   r_init_library_session();
   r_init_library_squash();
   r_init_library_stack();
+  r_init_library_vec();
   r_init_library_vec_chr();
 
   r_shared_true = r_new_vector(r_type_logical, 1);
