@@ -18,7 +18,7 @@ test_that("with_abort() promotes base errors to rlang errors", {
     )
   }
 
-  scoped_options(
+  local_options(
     rlang_trace_format_srcrefs = FALSE,
     rlang_trace_top_env = current_env()
   )
@@ -71,7 +71,7 @@ test_that("with_abort() entraces conditions properly", {
     }
   }
 
-  scoped_options(
+  local_options(
     rlang_trace_top_env = current_env()
   )
 
@@ -158,9 +158,28 @@ test_that("cnd_entrace() skips capture context", {
   foo <- function() bar()
   bar <- function() stop("foobar")
 
-  scoped_options(rlang_trace_top_env = current_env())
+  local_options(rlang_trace_top_env = current_env())
   err <- capture(foo())
 
   last <- err$trace$calls[[4]]
   expect_match(deparse(last), "bar")
 })
+
+test_that("rlang and base errors are properly entraced", {
+  skip_if_stale_backtrace()
+
+  base <- run_script(test_path("fixtures", "error-entrace.R"))
+
+  rlang <- run_script(
+    test_path("fixtures", "error-entrace.R"),
+    envvars = "rlang_error_kind=rlang"
+  )
+
+  verify_output(test_path("test-entrace.txt"), {
+    "# base error"
+    cat_line(base)
+
+    "# rlang error"
+    cat_line(rlang)
+  })
+})
